use std::path::Path;

use serde::Serialize;

use crate::config::Config;
use crate::error::CustomError;
use crate::intermediate::IHeading;

use super::RenderObject;

#[derive(Debug, Serialize)]
#[serde(tag = "type")]
#[serde(rename = "heading")]
pub(crate) struct RenderHeading {
    level: organic::types::HeadlineLevel,
    title: Vec<RenderObject>,
}

impl RenderHeading {
    pub(crate) fn new<D: AsRef<Path>, F: AsRef<Path>>(
        config: &Config,
        output_directory: D,
        output_file: F,
        heading: &IHeading,
    ) -> Result<RenderHeading, CustomError> {
        let title = heading
            .title
            .iter()
            .map(|obj| RenderObject::new(config, &output_directory, &output_file, obj))
            .collect::<Result<Vec<_>, _>>()?;
        Ok(RenderHeading {
            level: heading.level,
            title,
        })
    }
}
